from mykonos.keywords.management_device import ManagementDevice
import types


class Setup():
    def __init__(self):
        pass

    def remove_dot_version(self, **settings):
        version_android = self.__device_check(**settings)

        if '.' in version_android:
            version_android = version_android.split('.')
            return int(version_android.pop(0))
        else:
            return int(version_android)

    def __device_check(self, **settings):

        if 'device' in settings:
            get_device = settings['device']

            return next(ManagementDevice().get_android_version(device=get_device))
        else:
            return next(ManagementDevice().get_android_version())


class Decorators(object):

    def __init__(self):
        pass

    def android_version(func):

        def wrapper(self, *argument, **settings):
            management_device = ManagementDevice()
            setup = Setup()
            get_version_android = management_device.get_android_version()

            if len(list(get_version_android)) == 1 or 'device' in settings:

                version = setup.remove_dot_version(**settings)

                if 'text' in settings:
                    text = settings['text']
                    del settings['text']

                    if int(version) >= 9:
                        text_name = text.upper()
                        func(self, text=text_name, *argument, **settings)
                    else:
                        func(self, text=text, *argument, **settings)
                else:
                    func(self, *argument, **settings)
            else:
                print('device is more than 1, define the device or check using parallel tests')

        return wrapper



from mykonos.core.core import Core
from mykonos.keywords.management_device import ManagementDevice
from mykonos.keywords.decorators import Decorators

class GlobalElement(Core):
    def __init__(self):
        self.device_mobile = self.device()
        self.get = GetConditions()

    def open_notification(self, **settings):
        """Open notification a device.

        This keywords is used to open notification of device

        **Example:**

        || Open notification      |

        """

        if 'device' in settings:
            device = settings['device']
            return device.open.notification()
        else:
            return self.device_mobile.open.notification()

    def open_quick_settings(self, **settings):
        """Open Quick Setting a device.

        This keywords is used to open setting of device

        **Example:**

        || Open Quick setting      |

        """
        if 'device' in settings:
            device = settings['device']
            return device.open.quick_settings()
        else:
            return self.device_mobile.open.quick_settings()

    @Decorators.android_version
    def clear_text(self, *argument, **settings):
        """Clear text on the text field base on locator.

        This keywords is used to clear text field.

        **Example:**

        ||Clear Text        |  className=sample class
        """
        if 'locator' in settings:
            locator = settings['locator']
            return locator.clear_text()
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']
                return device(*argument, **settings).clear_text()
            else:
                return self.device_mobile(*argument, **settings).clear_text()

    @Decorators.android_version
    def input_text(self, *argument, **settings):
        """Input text on the text field base on locator.

        This keywords is used to input text into text field.

        **Example:**

        || Input Text        |  className=sample class    input=text

        """
        input = settings['input']
        del settings['input']

        if 'locator' in settings:
            locator = settings['locator']
            return locator.set_text(input)
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']

                return device(*argument, **settings).set_text(input)
            else:
                return self.device_mobile(*argument, **settings).set_text(input)

    @Decorators.android_version
    def count_elements(self, *argument, **settings):
        """Count total element from the page.

        This keywords is used to count total element on the device page.

        **Example:**

        || Count Elements          |  className=sample class

        **Return:**

        Total of elements (int)

        """
        if 'locator' in settings:
            locator = settings['locator']
            return locator.count
        elif 'watcher' in settings:
            watcher = settings['watcher']
            del settings['watcher']

            return watcher.count
        else:
            return self.device_mobile(*argument, **settings).count

    def turn_on_screen(self, **settings):
        """Turn on Screen Device.

        **Example:**

        ||  Turn On Screen

        **Return:**

         True or False
        """
        return self.device(**settings).screen.on()

    def turn_off_screen(self, **settings):
        """Turn off Screen Device.

        **Example:**

        ||  Turn Off Screen

        **Return:**

         True or False
        """
        return self.device(**settings).screen.off()

    def dump_xml(self, *args):
        """Dump hierarchy of ui and will be saved as hierarchy.xml.

        **Example:**

        ||  Dump Xml

        **Return:**

        xml file of device
        """
        return self.device().dump(*args)

    def capture_screen(self, file=None):
        """Capture screen of device testing.

        **Example:**

        ||  Capture Screen

        With file name:

        || Capture Screen        | file=sample

        **Return:**

        screen capture of device(*.png)
        """
        if file is not None:
            return self.device().screenshot(file+'.png')
        else:
            self.index += 1
            filename = 'mykonos-screenshot-%d.png' % self.index
            return self.device().screenshot(filename)


class Click(Core):

    def __init__(self):
        self.device_mobile = self.device()
        self.management_device = ManagementDevice()

    @Decorators.android_version
    def click_element(self, *argument, **settings):
        """Click on UI base on locator.

        This keyword is used to click button or element of device.

        **Example:**

        ||  Click Element                    | className=sample class
        """

        if 'locator' in settings:
            locator = settings['locator']
            return locator.click()

        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']

                get_device = self.management_device.scan_current_device(device)

                return get_device(*argument, **settings).click()

            elif 'watcher' in settings:
                watcher = settings['watcher']
                del settings['watcher']

                return watcher.click(*argument, **settings)
            else:
                print(settings)
                return self.device_mobile(*argument, **settings).click()

    @Decorators.android_version
    def long_click_element(self, *argument, **settings):
        """Long click on UI base on locator.

        This keyword is used to long click button or element of device.


        **Example:**

        ||  Long Click Element                 | className=sample class
        """
        if 'locator' in settings:
            locator = settings['locator']
            return locator.long_click()
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']
                return device(*argument, **locator).long_click()
            else:
                return self.device_mobile(*argument, **settings).long_click()

    def click_a_point(self, *argument, **settings):
        """Click into pointer target location.

         This keyword is used to click location  based on pointer X and Y.

         **Example:**

         ||  CLick A Point      |x=10   |y=20

        """
        if 'x' in settings and 'y' in settings:
            x = settings['x']
            y = settings['y']
            del settings['x']
            del settings['y']
        else:
            raise ValueError('pointer x or y is refused')

        if 'device' in settings:
            device = settings['device']
            del settings['device']
            return device(*argument, **settings).click(x, y)
        else:
            return self.device_mobile().click(x, y)


class GetConditions(Core):
    def __init__(self):
        self.device_mobile = self.device()

    def get_text(self, *argument, **settings):
        """Get text from element base on locator.

        **Example:**

        ||  Get Text         |  className=sample class

        **Return:**
        String
        """
        if 'locator' in settings:
            locator = settings['locator']
            return locator.info['text']
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']

                return device(*argument, **settings).info['text']
            else:
                return self.device_mobile(*argument, **settings).info['text']

    def get_element_attribute(self, *argument, **settings):
        """Get element attribute keyword of device.

        **List of Elements:**

         childCount, bounds, className, contentDescription,
         packageName, resourceName, text, visibleBounds,
         checkable, checked, clickable, enabled, focusable, disable,
         focused, longClickable, scrollable, selected

         **Example:**

         || Get Element Attribute         |  className=sample  element=text

         **Return:**

         Attribute from element device
        """
        element = settings['element']
        del settings['element']

        if 'locator' in settings:
            locator = settings['locator']
            return locator.info['element']
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']

                return device(*argument, **settings).info[element]
            else:
                return self.device_mobile(*argument, **settings).info[element]

    def get_element(self, *argument, **settings):
        """Get element info of device .
        This keyword is used to get element info of device.

         **Example:**

        || Get Element    |

        **Return:**

        {'currentPackageName': 'com.google.android.apps.nexuslauncher',
         'displayHeight': 1794,
         'displayRotation': 0,
         'displaySizeDpX': 411,
         'displaySizeDpY': 731,
         'displayWidth': 1080,
         'productName': 'sdk_google_phone_x86',
         'screenOn': True,
         'sdkInt': 25,
         'naturalOrientation': True}
        """
        if 'device' in settings:
            device = settings['device']
            del settings['device']

            return device(*argument, **settings).info
        else:
            return self.device_mobile(*argument, **settings).info

    def get_element_by_coordinate_x(self, *argument, **settings):
        """Get element by coordinate X.
        This keyword is used to get coordinate X of device.

         **Example:**

        || Get Element By Coordinate X  |  className=sample class

        **Return:**

        Coordinate x(int)
        """
        bound = self.get_conditions.get_element_attribute(element='bounds', *argument, **settings)

        bottom = bound['bottom']
        top = bound['top']
        elm_x = (top+bottom)+top

        return elm_x

    def get_element_by_coordinate_y(self, *argument, **settings):
        """Get element by coordinate Y.

        This keyword is used to get coordinate Y of device.

         **Example:**

        || Get Element By Coordinate Y  |  className=sample class

        **Return:**

        Coordinate y(int)
        """
        bound = self.get_element_attribute(element='bounds', *argument, **settings)
        display_height = self.get_height()
        height = display_height
        left = bound['left']
        right = bound['right']
        elm_y = height-(right+left)
        return elm_y

    def get_width(self):
        """Get width from display of device.

        This keyword is used to get widh of device,

        **Example:**

        || Get Width

        **Return:**

        Width of device(int)
        """
        get_device = self.device()
        return get_device.info['displayWidth']

    def get_height(self):
        """Get height from display of device.

        This keyword is used to get widh of device.

        **Example:**

        || Get Height

        **Return:**

        Width of device(int)
        """
        get_device = self.device()
        return get_device.info['displayHeight']


class ExpectedConditions(Core):
    def __init__(self):
        self.device_mobile = self.device()
        self.get_conditions = GetConditions()

    def page_should_contain_element(self, *argument, **settings):
        """Page should contain element.
        The keyword is used to verify the page is contains locator element.

        **Example:**

        || Page Should Contain Element | className=sample class

        **Return:**

        True or False
        """
        # element = self.get_conditions.get_element(*argument, **settings)

        if 'locator' in settings:
            locator = settings['locator']
            if locator.exists:
                return True
            else:
                raise ValueError('locator not found')
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']
                return device(*argument, **settings).exists
            else:
                return self.device_mobile(*argument, **settings).exists

    def page_should_contain_text(self, *argument, **settings):
        """Page should contain text.
        The keyword is used to verify the page is contains text.

        **Example:**

        || Page Should Contain Text | text=sample text

        **Return:**

        True or False
        """
        text = settings['text']

        if 'device' in settings:
            device = settings['device']
            del settings['device']
            return device(*argument, **settings).exists
        else:
            return self.device_mobile(*argument, **settings).exists

    def __get_device_global(self, *argument, **settings):
        if 'locator' in settings:
            device = settings['locator']
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']

                device = device(*argument, **settings)
            else:
                device = self.device_mobile(*argument, **settings)

        return device


    def page_should_not_contain_element(self, *argument, **settings):
        """Page should not contain element.

        The keyword is used to verify the page is not contains element.

        **Example:**

        || Page Should Not Contain Element | className=sample element

        **Return:**

        True or False
        """
        # element = self.get_conditions.get_element(*argument, **settings)

        if 'locator' in settings:
            locator = settings['locator']
            found = locator.exists
            if found is False:
                return True
            else:
                return False
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']
                found = device(*argument, **settings).exists
                if found is True:
                    return False
                else:
                    return True
            else:
                found = self.device_mobile(*argument, **settings).exists
                if found is True:
                    return False
                else:
                    return True

    def page_should_not_contain_text(self, *argument, **settings):
        """Page should not contain text.

        The keyword is used to verify the page is not contains text.

        **Example:**

        || Page Should Contain Text | text=sample text

        **Return:**

        True or False

        """
        text = self.get_conditions.get_text(*argument, **settings)

        if 'locator' in settings:
            locator = settings['locator']
            found = locator[text].exists
            if found is False:
                return True
            else:
                return False
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']
                return device(*argument, **settings).exists
            else:
                return self.device_mobile(*argument, **settings).exists


    def text_should_be_enabled(self, *argument, **settings):
        """Text should be enabled.

        The keyword is used to identify text enable.

        **Example:**

        || Text Should Be Enabled | text=sample text

        **Return:**

        True or False
        """
        element = self.get_conditions.get_element()
        enabled = element['enabled']
        if 'locator' in settings:
            locator = settings['locator']
            if locator.info['enabled'] is True:
                return True
            else:
                return False
        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            return self.device_mobile(*argument, **settings).enabled

    def text_should_be_disabled(self, *argument, **settings):
        """Text should be disabled.

        The keyword is used to identify text disabled.

        **Example:**

        || Element Should Be Disabled | text=sample text

        **Return:**

        True or False
        """
        element = self.get_conditions.get_element()
        enabled = element['enabled']
        if 'locator' in settings:
            locator = settings['locator']
            if locator.info['enabled'] is False:
                return True
            else:
                return False
        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            return self.device_mobile(*argument, **settings).enabled

    def element_should_contain_text(self, *argument, **settings):
        """Element should contain text.

        The keyword is used to identify text on element.

        **Example:**

        || Element Should Contain Text | className=class | text=sample text

        **Return:**

        True or False
        """
        if 'locator' in settings:
            locator = settings['locator']

            try:
                if locator.info['text'] is not None:
                    return True
            except Exception:
                return False
        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            try:
                if self.device_mobile(*argument, **settings).info['text'] is not None:
                    return True
            except Exception:
                return False

    def element_should_not_contain_text(self, *argument, **settings):
        """Element should contain text.

        The keyword is used to identify text on element.

        **Example:**

        || Element Should Not Contain Text | className=class | text=sample text

        **Return:**

        True or False
        """
        if 'locator' in settings:
            locator = settings['locator']

            try:
                if locator.info['text'] is None:
                    return True
                return False
            except Exception as error:
                return True

        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            try:
                if self.device_mobile(*argument, **settings).info['text'] is None:
                    return True
                return False
            except Exception as error:
                return True

    def check_element_visible(self, *argument, **settings):
        """Check element visible.

        The keyword is used to check element visible.

        **Example:**

        || Check Element Visible | className=sampleclassName

        **Return:**

        True or False
        """
        if 'locator' in settings:
            locator = settings['locator']

            try:
                if locator.info['visibleBounds'] is None:
                    return False
                return True
            except Exception as error:
                return ("Exception Error: {0}".format(error))

        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            try:
                if self.device_mobile(*argument, **settings).info['visibleBounds'] is None:
                    return False
                return True
            except Exception as error:
                return ("Exception Error: {0}".format(error))

    def check_element_non_visible(self, *argument, **settings):
        """Check element non visible.

        The keyword is used to check element non visible.

        **Example:**

        || Check Element Non Visible | className=sampleclassName

        **Return:**

        True or False
        """
        if 'locator' in settings:
            locator = settings['locator']

            try:
                if locator.info['visibleBounds'] is None:
                    return True
                return False
            except Exception as error:
                return ("Exception Error: {0}".format(error))

        else:
            if 'device' in settings:
                device_mobile = settings['device']
                del settings['device']

            try:
                if self.device_mobile(*argument, **settings).info['visibleBounds'] is None:
                    return True
                return False
            except Exception as error:
                return ("Exception Error: {0}".format(error))



from mykonos.core.core import Core
from mykonos.keywords.decorators import Decorators

class Touch(Core):
    def __init__(self):
        self.device_mobile = self.device()

    def __get_device_global(self, *argument, **settings):
        if 'locator' in settings:
            device = settings['locator']
        else:
            if 'device' in settings:
                device = settings['device']
                del settings['device']

                device = device(*argument, **settings)
            else:
                device = self.device_mobile(*argument, **settings)

        return device

    def __get_device_scroll(self, *argument, **settings):
        if 'device' in settings:
            device = settings['device']
            del settings['device']
            return device(scrollable=True)
        else:
            return self.device_mobile(scrollable=True)

    def __check_action_device_scroll(self, *argument, **settings):
        device = self.__get_device_scroll(self, *argument, **settings)

        if 'action' in settings:
            action = settings['action']
            del settings['action']

            if 'horizontal forward' in action:
                return device.scroll.horiz.forward(**settings)
            elif 'horizontal to begining' in action:
                return device.scroll.horiz.toBeginning(**settings)
            elif 'horizontal backward' in action:
                return device.scroll.horiz.backward(**settings)
            elif 'horizontal to end' in action:
                return device.scroll.horiz.toEnd(**settings)
            elif 'horizontal to' in action:
                return device.scroll.horiz.to(**settings)
            elif 'vertical backward' in action:
                return device.scroll.vert.backward(**settings)
            elif 'vertical to end' in action:
                return device.scroll.vert.toEnd(**settings)
            elif 'vertical forward' in action:
                return device.scroll.vert.forward(**settings)
            elif 'vertical to begining' in action:
                return device.scroll.vert.toBeginning(**settings)
            elif 'vertical to' in action:
                return device.scroll.vert.to(**settings)
            else:
                raise Exception('Action is not available on ui automator')
        else:
            return self.__get_device_scroll(self, *argument, **settings).scroll(steps=10)

    def __check_action_device_fling(self, *argument, **settings):
        device = self.__get_device_scroll(self, *argument, **settings)

        if 'action' in settings:
            action = settings['action']
            del settings['action']

            if 'horizontal forward' in action:
                return device.fling.horiz.forward(max_swipes=10)
            elif 'horizontal to begining' in action:
                return device.fling.horiz.toBeginning(max_swipes=10)
            elif 'horizontal backward' in action:
                return device.fling.horiz.backward(max_swipes=10)
            elif 'horizontal to end' in action:
                return device.fling.horiz.toEnd(max_swipes=10)
            elif 'vertical backward' in action:
                return device.fling.vert.backward(max_swipes=10)
            elif 'vertical to end' in action:
                return device.fling.vert.toEnd(max_swipes=10)
            elif 'vertical forward' in action:
                return device.fling.vert.forward(max_swipes=10)
            elif 'vertical to begining' in action:
                return device.fling.vert.toBeginning(max_swipes=10)
            else:
                raise Exception('Action is not available on ui automator')
        else:
            return self.__get_device_scroll(self, *argument, **settings).fling()

    def swipe(self, sx, sy, ex, ey, steps, **settings):
        """Geasture swipe with interanction on device.

        Swipe from (sx, sy) to (ex, ey).

        **Example:**

        || Swipe        | sx=10  sy=10  ex=20   ey=20   |  steps=100

        """

        if 'device' in settings:
            device = settings['device']

            del settings['device']
            return device.swipe(sx, sy, ex, ey, steps)
        else:
            return self.device_mobile.swipe(sx, sy, ex, ey, steps)

    def swipe_with_direction(self, *argument, **settings):
        """Gesture swipe with direction on device.

        Swipe with direction : right, left, up and down

        **Example:**

        ||Swipe                             | direction=right   |  steps=100

        ||Swipe                             | direction=left    |  steps=100

        ||Swipe                             | direction=up      |  steps=100

        ||Swipe                             | direction=down    |  steps=100
        """
        direction = settings['direction']
        del settings['direction']

        if 'steps' in settings:
            steps = settings['steps']
            del settings['steps']

        dvc = self.__get_device_global(*argument, **settings)

        if 'right' in direction:
            return dvc.swipe.right(steps=1)
        elif 'left' in direction:
            return dvc.swipe.left(steps=1)
        elif 'up' in direction:
            return dvc.swipe.up(steps=1)
        elif 'down' in direction:
            return dvc.swipe.down(steps=1)

    def drag_screen(self, sx, sy, ex, ey, steps, *argument, **settings):
        """Geasture drag interanction on device.

        This keyword is used to drag another point ui object to another point ui object.

        **Example:**

        || Drag Screen    | sx=189 | sy=210 | ex=954 | ey=336   |  steps=100
        """
        device = settings['device']

        if device is not None:
            return device(*argument, **settings).drag(sx, sy, ex, ey, steps)
        else:
            return self.device_mobile().drag(sx, sy, ex, ey, steps)

    @Decorators.android_version
    def scroll(self, *argument, **settings):
        """Scroll interanction on device.

        This keyword is used to perfom scroll on device.

        **Example:**

        How to use scroll horizontal:

        || Scroll     | steps=100

        || Scroll     | steps=100     action=horizontal forward

        || Scroll     | steps=100  max_swipes=1   action=horizontal to begining

        || Scroll     | textName='Calculator' clasName='sampleClass'    action=horizontal to
        || Scroll     | action=horizontal backward

        || Scroll     | action=horizontal to end

        How to use scroll vertical:

        || Scroll    | steps=100

        || Scroll    | steps=100      action=vertical forward

        || Scroll    | steps=100 max_swipes=1     action=vertical to begining

        || Scroll    | textName='Calculator' clasName='sampleClass'       action=vertical to

        || Scroll    | action=vertical backward

        || Scroll    | action=vertical to end

         """
        return self.__check_action_device_scroll(self, *argument, **settings)

    @Decorators.android_version
    def pinch(self, *argument, **settings):
        """Pinch interaction on Device

        **Example:**

        || Pinch      | steps=100     action=In    percent=100

        || Pinch      | steps=100     action=Out   percent=100

        """

        if 'percent' in settings:
            percent = settings['percent']

            del settings['percent']

        if 'steps' in settings:
            steps = settings['steps']

            del settings['steps']

        if 'action' in settings:
            action = settings['action']
            del settings['action']

            device = self.__get_device_global(*argument, **settings)

            if 'In' in action:
                return device.pinch.In(percent=10, steps=10)
            elif 'Out' in action:
                return device.pinch.Out(percent=10, steps=10)
        else:
            raise Exception('Action is not available on ui automator')

    @Decorators.android_version
    def fling(self, *argument, **settings):
        """Fling interanction on Android device.

        This keyword is used to perform fling to spesific ui object.

        **Example:**

        How to user without action:

        || Fling

        How to use fling horizontal:

        || Fling            | action=horizontal forward

        || Fling            | max_swipes=1   action=horizontal to begining

        || Fling            | action=horizontal backward

        || Fling            | action=horizontal to end

        How to use scroll vertical:

        || Fling            | action=vertical forward

        || Fling            | max_swipes=1   action=vertical to begining

        || Fling            | action=vertical backward

        || Fling            | action=vertical to end

         """
        return self.__check_action_device_fling(self, *argument, **settings)
